function [repro_dist, surv_dist, repr_lag] = Escape(esc_day);

%  Andi Stephens, April 2004
%  Re-coding in MATLAB of Andy Cooper's Lipid-based reproduction model
%  for escaped aquaculture salmon.


global predday1 predday2 beginwin1 beginwin2 triggerday1 triggerday2 window1 window2
global Weight actWeight G1 G2 q wildq wildq_penalty qe qi_stdev initialWt_stdev genetic_initialWt
global Length actLength Lipid actLipid k cumk cumk2 firstday initialWt aquaculture_genetic_q
global wild_q_factor qi_mean qi_cv qi reprDay
global daycounter yrDay totDay Winter anorexday1 anorexcost
global Sigma actSigma mort cummort cumsurv gonads m1 m2 reproduction
global realmort realcummort realcumsurv realgonads
global predWeight predLength predLipid
global gonads foodassim wtMax basemort slopemort growth_penalty
global matureCr EscapeDay EscapeEffect 

single_escape = 1;

dat_EscapeDay = esc_day;
dat_Escape_Interval = 50;

firstday = 84;
lastday = 4000;
anorexday1 = 197;

window1 = 21;
window2 = 21;
triggerday1 = 315;
triggerday2 = 106;

Lmax = 150;
wtMax = exp((log(Lmax) - 1.556)/0.323);

matureCr = 0.988;
anorexcost = 0.001;
growth_penalty = 0.08;

basemort = 0.00354;
slopemort = 0.014;

penalty = .8;

EscapeEffect = .75;

aquaculture_genetic_q = 0.08;

wild_q_factor = 0.9;

aggression_penalty = 0.8;

dat_qi_mean = 0.08;
qi_mean = 0.08;
qi_cv = 0.2;
qi_stdev = dat_qi_mean * qi_cv;

genetic_initialWt = 47;

initialWt_mean = 47;
initialWt_cv = 0.3;


% qi = (0.0001,3,var_qi);
% initialWt = (0.01,200,var_initialWt);

qi = qi_mean + qi_mean * qi_cv * randn(1);
initialWt = initialWt_mean + initialWt_mean * initialWt_cv * randn(1);


Weight = zeros(1,lastday);
actWeight = zeros(1,lastday);
predWeight = zeros(1,lastday);
reprWeight = zeros(1,lastday);

Length = zeros(1,lastday);
actLength = zeros(1,lastday);
predLength = zeros(1,lastday);

Lipid = zeros(1,lastday);
actLipid = zeros(1,lastday);
predLipid = zeros(1,lastday);
reprLipid = zeros(1,lastday);

mort = zeros(1,lastday);
cummort = zeros(1,lastday);
cumsurv = zeros(1,lastday);

realmort = zeros(1,lastday);
realcummort = zeros(1,lastday);
realcumsurv = zeros(1,lastday);
endsurvival = zeros(1,lastday);

gonads = zeros(1,lastday);
finalgonad = zeros(1,lastday);
realgonads = zeros(1,lastday);
real_reprod = zeros(1,lastday);

Sigma = zeros(1,685);
actSigma = zeros(1,685);
k = zeros(1,lastday);
cumk = zeros(1,lastday);
cumk2 = zeros(1,lastday);

qe = zeros(1,lastday);
temp = zeros(1,lastday);
foodassim = zeros(1,lastday);

% function get_temp_trend

for i=1:lastday
    temp(i) = 10.5 + 2 * cos((i-210) * 2 * pi / 365);
    foodassim(i) = 1 - ((temp(i) - 6) / 12 )^2;
end

%  Main Loop

f=0;

get_set_timeframe;
get_initial_variables;
get_estimate_m1_m2;

%%  REALITY

if (single_escape == 1)
    nescapes = 0;
else
    nescapes = (600/dat_Escape_Interval);
end


for (escday = 0:nescapes)
    
    get_set_timeframe;
    get_initial_variables;
    
    if (single_escape ==1)
        EscapeDay = dat_EscapeDay;
    else
        EscapeDay = firstday + dat_Escape_Interval*escday;
    end
    
    daycounter = firstday;
    yrDay = firstday;
    totDay = firstday;
    Winter = 0;
    
    while ((G2==0) && (totDay <= lastday-209))
        
        get_count_days;
        get_escape_effect;
        get_grow_while_immature;
        
        if ((totDay >= beginwin2) && (totDay <= triggerday2) && (G1 == 1))
            get_g2_assess_window;
        end
        
        if ((totDay == triggerday2) && (G1 == 1))
            get_m2_projection;
        end
        
        if ((totDay >= beginwin1) && (totDay <= triggerday1) && (G2 == 0))
            get_g1_assess_window;
        end
        
        if ((totDay == triggerday1) && (G2 == 0))
            get_m1_projection;
        end
        
        get_mortality_imm;
    end
    
    
    for (matDay = daycounter + 1:reprDay(EscapeDay))
        get_grow_while_mature;
    end
end

initialWt_stdev = initialWt_mean * initialWt_cv;
f = (qi - dat_qi_mean)^2/qi_stdev + (initialWt - initialWt_mean)^2/initialWt_stdev;

expected_repro = cumsurv .* gonads;
repro_dist = expected_repro / sum(expected_repro);

surv_dist = cumsurv / sum(cumsurv);

repr_lag = round((mean(nonzeros(reprDay)) - esc_day)/365);

%%%%%%%%%%%%%%%%%%%%%  Functions Start Here%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%********************** Timeframe*******************

function get_set_timeframe

global predday1 predday2 beginwin1 beginwin2 triggerday1 triggerday2 window1 window2

beginwin1 = triggerday1 - window1;
beginwin2 = triggerday2 - window2;
predday1 = triggerday1 + 156;
predday2 = triggerday2 + 209;

% end function get_set_timeframe

%********************** Initialize Globals *******************

function get_initial_variables

global Weight actWeight G1 G2 q wildq qe qi_stdev initialWt_stdev genetic_initialWt
global Length actLength Lipid actLipid cumk cumk2 firstday aquaculture_genetic_q
global qi_mean qi_cv initialWt_mean initialWt_cv wild_q_factor beginwin1 beginwin2
global initialWt


Weight(firstday) = genetic_initialWt;
actWeight(firstday) = initialWt;

G1 = 1;
G2 = 0;

q = aquaculture_genetic_q;
wildq = q * wild_q_factor;

qe(firstday) = 1;
qi_stdev = qi_mean*qi_cv;

initialWt_stdev = initialWt_mean*initialWt_cv;

Length(firstday) = exp(1.613 + 0.312 * log(Weight(firstday)));
actLength(firstday) = Length(firstday);

Lipid(firstday) = -0.82 + 0.00418 * Weight(firstday) + 0.000199 * Length(firstday);
actLipid(firstday) = Lipid(firstday);

cumk(beginwin1 - 1) = 0;
cumk2(beginwin2 - 1) = 0;

% end function get_initial_variables

%********************** Get Count Days *******************

function get_count_days

global predday1 predday2 beginwin1 beginwin2 triggerday1 triggerday2
global k cumk cumk2 daycounter yrDay totDay Winter anorexday1

totDay = totDay + 1;
yrDay = yrDay + 1;
daycounter = daycounter + 1;

if (yrDay > 365)
    
    yrDay = 1;
    Winter = Winter + 1;
    triggerday1 = triggerday1 + 365;
    triggerday2 = triggerday2 + 365;
    beginwin1 = beginwin1 + 365;
    beginwin2 = beginwin2 + 365;
    
    cumk(beginwin1 - 1) = 0;
    cumk2(beginwin2 - 1) = 0;
    
    predday1 = predday1 + 365;
    predday2 = predday2 + 365;
    anorexday1 = anorexday1 + 365;
end

% end function get_count_days

%********************** Estimate M1 and M2 *******************

function get_estimate_m1_m2

global triggerday1 triggerday2 firstday totDay EscapeEffect
global Weight q qe matureCr anorexcost anorexday1
global gonads wtMax foodassim basemort slopemort
global Length Lipid actLipid k cumk cumk2 predday1 predday2
global Sigma mort cummort cumsurv gonads m1 m2 reproduction

for t = firstday+1:680
    if t < (anorexday1 + 365)
        if t < (triggerday2 + 365)
            Weight(t) = ((Weight(t-1))^(1/3) + (1/3 * q * foodassim(t)))^3;
        else
            Weight(t) = ((Weight(t-1))^(1/3) + (1/3 * q * matureCr * foodassim(t)))^3;
        end
    else
        Weight(t) = Weight(t-1) * (1-anorexcost);
    end
    
    if (Weight(t) > wtMax)
        Weight(t) = wtMax;
    end
    
    if (t < triggerday2 + 365)
        Length(t) = exp(1.613 + 0.312 * log(Weight(t)));
    else
        Length(t) = exp(1.613 + 0.323 * log(Weight(t)));
    end
    
    if (Length(t) < Length(t-1))
        Length(t) = Length(t-1);
    end
    
    Lipid(t) = -0.82 + 0.00418 * Weight(t) + 0.000199 * Length(t);
    
    Sigma(t) = Weight(t-1)^(-2/3) * (Weight(t) - Weight(t-1));
    
    mort(t) = basemort + (slopemort * Weight(t)^(-.37));
    cummort(t) = cummort(t-1) + mort(t);
    cumsurv(t) = exp(-1 * cummort(t));
    gonads(t) = -0.326 + 0.194 * Weight(t);
    
    if (gonads(t) < 0)
        gonads(t) = 0;
    end
end

m1 = Lipid(predday1);
m2 = Lipid(predday2 + 365);

reproduction = cumsurv(680) * gonads(680);

% end function get_estimate_m1_m2

%********************** Get Escape Effect  *******************

function get_escape_effect

global totDay EscapeDay qe firstday EscapeEffect wildq_penalty

if (totDay >= EscapeDay)
    qe(totDay) = qe(firstday) * EscapeEffect;
else 
    qe(totDay) = qe(firstday);
    wildq_penalty = 0;
end

% end function get_escape_effect

%********************** Mature Growth *******************

function get_grow_while_immature

global totDay foodassim qi qe actWeight actLength actLipid wtMax

actWeight(totDay) = ((actWeight(totDay - 1)^(1/3)) + 0.33334 * qi * qe(totDay) * foodassim(totDay))^3;

if (actWeight(totDay) > wtMax)
    actWeight(totDay) = wtMax;
end

actLength(totDay) = exp(1.613 + 0.312 * log(actWeight(totDay)));

if (actLength(totDay) < actLength(totDay - 1))
    actLength(totDay) = actLength(totDay - 1);
end

actLipid(totDay) = -0.82 + 0.00418 * actWeight(totDay) + 0.000199 * actLength(totDay);

% end function get_grow_while_immature

%********************** G2 Assessment window *******************

function get_g2_assess_window

global Sigma actSigma totDay actWeight k yrDay cumk2 triggerday2 beginwin2


actSigma(totDay) = actWeight(totDay - 1)^(-2/3) * (actWeight(totDay) - actWeight(totDay - 1));
k(totDay) = actSigma(totDay) / (Sigma(450 + yrDay-85));
cumk2(totDay) = cumk2(totDay - 1) + (k(totDay)/(triggerday2 - beginwin2));
    
% end function get_g2_assess_window

%********************** M2 Projection *******************

function get_m2_projection

global predWeight totDay actWeight predLength actLength predLipid actLipid
global predday2 anorexday1 cumk2 q matureCr foodassim anorexcost wtMax
global m2 reprDay EscapeDay G2

predWeight(totDay) = actWeight(totDay);
predLength(totDay) = actLength(totDay);
predLipid(totDay) = actLipid(totDay);

for (time5 = totDay + 1:predday2 )
    
    if (time5 <= anorexday1)
        predWeight(time5) = (predWeight(time5 - 1)^(1/3) + 0.3333333 * cumk2(totDay) ...
            * q * matureCr * foodassim(time5))^3;
    else
        predWeight(time5) = predWeight(time5 - 1) * (1 - anorexcost);
    end
    
    if(predWeight(time5) > wtMax)
        predWeight(time5) = wtMax;
    end
    
    predLength(time5) = exp(1.556 + 0.323 * log(predWeight(time5)));
    
    if(predLength(time5) < predLength(time5 - 1))
        predLength(time5) = predLength(time5 - 1);
    end
    
    predLipid(time5) = -0.82 + 0.00418 * predWeight(time5) + 0.000199 * predLength(time5);
end
    
if (predLipid(predday2) >= m2)
    reprDay(EscapeDay) = predday2;
    G2 = 1;
else 
    G2 = 0;
end
    
% end function get_m2_projection

%********************** M1 Assessment window *******************

function get_g1_assess_window

global Sigma actSigma totDay actWeight k yrDay cumk triggerday1 beginwin1

 actSigma(totDay) = actWeight(totDay - 1)^(-2/3) * (actWeight(totDay) - actWeight(totDay - 1));
 k(totDay) = actSigma(totDay) / (Sigma(yrDay));
 cumk(totDay) = cumk(totDay - 1) + (k(totDay)/(triggerday1 - beginwin1));
 
% end function get_g1_assess_window

%********************** M1 Projection *******************

function get_m1_projection

global predWeight predLength predLipid actWeight actLength actLipid totDay predday1
global cumk q foodassim wtMax G1 m1

predWeight(totDay) = actWeight(totDay);
predLength(totDay) = actLength(totDay);
predLipid(totDay) = actLipid(totDay);

for (time3 = totDay + 1:predday1)
    
    predWeight(time3) = (predWeight(time3 - 1)^(1/3) + 0.3333333 * cumk(totDay) * q * foodassim(time3))^3;
    
    if (predWeight(time3) > wtMax)
        predWeight(time3) = wtMax;
    end
    
    predLength(time3) = exp(1.613 + 0.312 * log(predWeight(time3)));
    
    if (predLength(time3) < predLength(time3 - 1))
        predLength(time3) = predLength(time3 - 1);
    end
    
    predLipid(time3) = -0.82 + 0.00418 * predWeight(time3) + 0.000199 * predLength(time3);
end

if (predLipid(predday1) >= m1)
    G1 = 1;
else 
    G1 = 0;
end

% end function get_m1_projection
       
%********************** Immature Mortality *******************

function get_mortality_imm

global totDay basemort slopemort actWeight growth_penalty qi q wildq_penalty wildq
global realmort realcummort realcumsurv

realmort(totDay) = (basemort + slopemort * actWeight(totDay)^(-0.37)) ...
    * (1 + growth_penalty * (qi - q) / q) * (1 + wildq_penalty * (qi - wildq) / wildq);

realcummort(totDay) = realcummort(totDay - 1) + realmort(totDay);
realcumsurv(totDay) = exp(-1 * realcummort(totDay));

% end function get_mortality_imm

%********************** Mature Growth *******************
    
function get_grow_while_mature

global matDay EscapeDay qe firstday EscapeEffect wildq_penalty dat_wildq_penalty wildq
global anorexday1 actWeight qi matureCr foodassim anorexcost q wtMax actLipid actLength
global basemort slopemort realmort realgonads realcummort realcumsurv growth_penalty
global dat_qi_mean

if (matDay >= EscapeDay)
    qe(matDay) = qe(firstday) * EscapeEffect;
    wildq_penalty = dat_wildq_penalty;
else 
    qe(matDay) = qe(firstday);
    wildq_penalty = 0;
end

if (matDay <= anorexday1)
    actWeight(matDay) = (actWeight(matDay - 1)^(1/3) + 0.3333333 * qi ...
        * qe(matDay) * matureCr * foodassim(matDay))^3;
else
    actWeight(matDay) = actWeight(matDay - 1) * (1 - anorexcost * qi / q);
end

if (actWeight(matDay) > wtMax)
    actWeight(matDay) = wtMax;
end

actLength(matDay) = exp(1.556 + 0.323 * log(actWeight(matDay)));

if (actLength(matDay) < actLength(matDay - 1))
    actLength(matDay) = actLength(matDay - 1);
end
    
actLipid(matDay) = -0.82 + 0.00418 * actWeight(matDay) + 0.000199 * actLength(matDay);

realmort(matDay) = (basemort + slopemort * actWeight(matDay)^(-0.37)) ...
    * (1 + growth_penalty * (qi - q) / q) * (1 + wildq_penalty * (qi - wildq) / wildq);

realcummort(matDay) = realcummort(matDay - 1) + realmort(matDay);
realcumsurv(matDay) = exp(-1 * realcummort(matDay));

realgonads(matDay) = -0.326 + 0.194 * actWeight(matDay);

if (realgonads(matDay) < 0)
    realgonads(matDay) = 0;
end

% end function get_grow_while_mature

    
